-- AdoptHarmony
-- Author: Hokath
-- DateCreated: 16/10/2024 10:53:27 AM
--------------------------------------------------------------
local iHarmony = GameInfoTypes["POLICY_BRANCH_HARMONY"]

-- this is a jfd creation
-- we will use for deciding if our leader is harmonious...
local g_Leader_Flavors_Table = {}
local g_Leader_Flavors_Count = 1
for row in DB.Query("SELECT * FROM Leader_Flavors;") do 	
	g_Leader_Flavors_Table[g_Leader_Flavors_Count] = row
	g_Leader_Flavors_Count = g_Leader_Flavors_Count + 1
end

local g_LeaderFlavours_Table = {}
for otherPlayerID = 0, GameDefines.MAX_MAJOR_CIVS - 1 do
	local otherPlayer = Players[otherPlayerID]
	if otherPlayer:IsAlive() then
		local leaderID = otherPlayer:GetLeaderType()
		local leader = GameInfo.Leaders[leaderID]
		local leaderType = leader.Type
		g_LeaderFlavours_Table[otherPlayerID] = {}
		--g_Leader_Flavors_Table
		local flavorsTable = g_Leader_Flavors_Table
		local numFlavors = #flavorsTable
		for index = 1, numFlavors do
			local row = flavorsTable[index]
			if row.LeaderType == leaderType then
				g_LeaderFlavours_Table[otherPlayerID][row.FlavorType] = row.Flavor
			end
		end
	end
end

function Player.GetFlavorValue(player, flavorType)
	return g_LeaderFlavours_Table[player:GetID()][flavorType] or 5
end	

-- this holds the logic of which ideology to prefer
-- it is currently very primitive and also RNG
function shouldSwitchHarmony(pPlayer)
	-- first define a multiplier that gets bigger the more people follow harmony
	local multiplier = 1.0
	for iAllPlayer = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do 
		local pAllPlayer = Players[iAllPlayer]
		if (pAllPlayer ~= pPlayer) and pAllPlayer:HasIdeology(iHarmony) then 	
			multiplier = multiplier + 0.25	-- 25% more likely
		end
	end
	print("the multiplier is " .. multiplier)
        -- get some important leader flavours
	-- but dont include too much or it dilutes the effect of each
	-- growth and defense are pretty similar for everyone, ignore those
	local fOffense  = pPlayer:GetFlavorValue("FLAVOR_OFFENSE") 
	local fScience  = pPlayer:GetFlavorValue("FLAVOR_SCIENCE") 
	local fNuke     = pPlayer:GetFlavorValue("FLAVOR_NUKE") 
	local fReligion = pPlayer:GetFlavorValue("FLAVOR_RELIGION") 
	local fWonder   = pPlayer:GetFlavorValue("FLAVOR_WONDER") 
	-- combine in a way that makes big numbers more harmony
	local fRange = fScience - fNuke + fReligion + fWonder - fOffense
	print("the fRange is " .. fRange)
	-- the max is therefore around 30 before the multiplier
	if (Game.Rand(30, "Harmony Choice") < (fRange * multiplier)) then
		return 1
	else
		return 0
	end
end

-- now the function
function AdoptHarmony(iPlayer, iPolicyBranch)
	-- this is just for AI. I think Poland has issues so block them for simplicity.
	-- for some reason this is called for non ideologies? 9 is first ideology hopefully
	local pPlayer = Players[iPlayer];
	if (iPolicyBranch < 9) or pPlayer:IsHuman() or (pPlayer:GetCivilizationType() == GameInfoTypes["CIVILIZATION_POLAND"]) then return false end
	-- if they were going autocracy, harmony is inappropriate
	if iPolicyBranch == GameInfoTypes["POLICY_BRANCH_AUTOCRACY"] then return false end
	-- so this unfortunately calls itself. what happens is you get 3 events.
	-- 1 when you adopt, this fires our function
        -- then 1 when you switch off ipolicybranch, we want to ignore this one
        -- then 1 when you switch on iharmony, this one actually does stuff below here
	print("argument policy branch id is " .. iPolicyBranch)
	-- if this was a revolution, we would want to ignore
        -- sadly this might fire before they choose tenets, so TODO later
	-- note the AI never adopts harmony themselves (hence this function!)
	local numTenets = pPlayer:GetNumPoliciesPurchasedInBranch(iPolicyBranch)
	if numTenets > 0 then
		print("there are tenets, abort!")
		return false
	end
	-- for the first time around
	if iPolicyBranch ~= iHarmony then
		-- if they were going for something with an ID bigger than 11, but isnt harmony, it was another mod (devotion?) and should ignore!
		if (iPolicyBranch > 11) then return false end
		-- now work out if the player should prefer harmony (we dont care about outcome of second call)
		local proceed = shouldSwitchHarmony(pPlayer)
		if proceed == 0 then return false end
		print("proceeding")
		-- this apparently only sends an event when it changes, so just 1 call from each
		pPlayer:SetPolicyBranchUnlocked(iPolicyBranch, false)  
		pPlayer:SetPolicyBranchUnlocked(iHarmony, true)
	end
	-- for the third time around
	if iPolicyBranch == iHarmony then
		print("AdoptHarmony successful")
		pPlayer:SetNumFreeTenets(1)  -- or they'll have extra
		-- notification
		local sText = pPlayer:GetName() .. " has been forced to adopt [COLOR_MAGENTA]Harmony[ENDCOLOR] due to a conspiracy of activists, spiritualists, and Hokaths Lua code!"
		for iAllPlayer = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do 
			local pAllPlayer = Players[iAllPlayer]
			sText = "Although they tried to follow a different path, " .. sText
			if pAllPlayer:IsHuman() then 	
				pAllPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, sText, "Ideological Coup!")
			end
		end
	end
end

GameEvents.IdeologyAdopted.Add(AdoptHarmony)